@@ -14,12 +14,10 @@
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public Lesser License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>
  *
  */
 
@@ -31,7 +29,13 @@
 #include "fsp-session.h"
 
 #include <config.h>
+
+#ifdef HAVE_LIBSOUP_GNOME
+#include <libsoup/soup-gnome.h>
+#else
 #include <libsoup/soup.h>
+#endif
+
 #include <stdarg.h>
 #include <string.h>
 
@@ -60,6 +64,7 @@ struct _FspSessionPrivate
   gchar *token;
   gchar *frob;
 
+  gboolean using_gnome_proxy;
   SoupURI *proxy_uri;
   SoupSession *soup_session;
 };
@@ -256,6 +261,10 @@ _get_tags_list_soup_session_cb           (SoupSession *session,
                                           SoupMessage *msg,
                                           gpointer     data);
 
+static void
+_set_license_soup_session_cb            (SoupSession *session,
+                                         SoupMessage *msg,
+                                         gpointer     data);
 
 /* Private API */
 
@@ -397,6 +406,7 @@ fsp_session_init                        (FspSession *self)
   self->priv->token = NULL;
   self->priv->frob = NULL;
 
+  self->priv->using_gnome_proxy = FALSE;
   self->priv->proxy_uri = NULL;
 
   self->priv->soup_session = soup_session_async_new ();
@@ -1242,6 +1252,19 @@ _get_tags_list_soup_session_cb           (SoupSession *session,
                          data);
 }
 
+static void
+_set_license_soup_session_cb            (SoupSession *session,
+                                         SoupMessage *msg,
+                                         gpointer     data)
+{
+  g_assert (SOUP_IS_MESSAGE (msg));
+  g_assert (data != NULL);
+
+  /* Handle message with the right parser */
+  _handle_soup_response (msg,
+                         (FspParserFunc) fsp_parser_set_license,
+                         data);
+}
 
 
 /* Public API */
@@ -1263,53 +1286,85 @@ fsp_session_new                         (const gchar *api_key,
 
 gboolean
 fsp_session_set_http_proxy              (FspSession *self,
+                                         gboolean use_gnome_proxy,
                                          const char *host, const char *port,
                                          const char *username, const char *password)
 {
   SoupURI *proxy_uri = NULL;
 
+#ifdef HAVE_LIBSOUP_GNOME
+  gboolean using_gnome_proxy_before = FALSE;
+#endif
+
   g_return_val_if_fail (FSP_IS_SESSION (self), FALSE);
 
-  if (host != NULL)
+#ifdef HAVE_LIBSOUP_GNOME
+  /* We're gonna need this to make a good decision later */
+  using_gnome_proxy_before = self->priv->using_gnome_proxy;
+
+  if (using_gnome_proxy_before != use_gnome_proxy)
     {
-      const gchar *actual_user = NULL;
-      const gchar *actual_password = NULL;
-      guint actual_port = 0;
-
-      /* Ensure no garbage is used for username */
-      if (username == NULL || *username == '\0')
-        actual_user = NULL;
+      void (*soup_feature_func) (SoupSession *, GType) = NULL;
+      if (use_gnome_proxy)
+        soup_feature_func = soup_session_add_feature_by_type;
       else
-        actual_user = username;
+        soup_feature_func = soup_session_remove_feature_by_type;
 
-      /* Ensure no garbage is used for password */
-      if (password == NULL || *password == '\0')
-        actual_password = NULL;
-      else
-        actual_password = password;
+      /* Add or remove the feature */
+      soup_feature_func (self->priv->soup_session,
+                         SOUP_TYPE_PROXY_RESOLVER_GNOME);
+    }
+  self->priv->using_gnome_proxy = use_gnome_proxy;
+#else
+  /* Just ignore the using_gnome_proxy parameter in this case */
+  self->priv->using_gnome_proxy = FALSE;
+#endif
 
-      /* We need a numeric port */
-      actual_port = (guint) g_ascii_strtoll ((gchar *) port, NULL, 10);
+  if (!self->priv->using_gnome_proxy)
+    {
+      /* If using the GNOME proxy we just forget about the rest. */
+      if (host != NULL)
+        {
+          const gchar *actual_user = NULL;
+          const gchar *actual_password = NULL;
+          guint actual_port = 0;
 
-      /* Build the actual SoupURI object */
-      proxy_uri = soup_uri_new (NULL);
-      soup_uri_set_scheme (proxy_uri, SOUP_URI_SCHEME_HTTP);
-      soup_uri_set_host (proxy_uri, host);
-      soup_uri_set_port (proxy_uri, actual_port);
-      soup_uri_set_user (proxy_uri, actual_user);
-      soup_uri_set_password (proxy_uri, actual_password);
+          /* Ensure no garbage is used for username */
+          if (username == NULL || *username == '\0')
+            actual_user = NULL;
+          else
+            actual_user = username;
+
+          /* Ensure no garbage is used for password */
+          if (password == NULL || *password == '\0')
+            actual_password = NULL;
+          else
+            actual_password = password;
+
+          /* We need a numeric port */
+          actual_port = (guint) g_ascii_strtoll ((gchar *) port, NULL, 10);
+
+          /* Build the actual SoupURI object */
+          proxy_uri = soup_uri_new (NULL);
+          soup_uri_set_scheme (proxy_uri, SOUP_URI_SCHEME_HTTP);
+          soup_uri_set_host (proxy_uri, host);
+          soup_uri_set_port (proxy_uri, actual_port);
+          soup_uri_set_user (proxy_uri, actual_user);
+          soup_uri_set_password (proxy_uri, actual_password);
+        }
     }
 
-  /* Set/unset the proxy if needed */
+  /* Set/unset the proxy */
+  g_object_set (G_OBJECT (self->priv->soup_session),
+                SOUP_SESSION_PROXY_URI,
+                proxy_uri,
+                NULL);
+
+  /* Update internal values if needed */
   if ((!self->priv->proxy_uri && proxy_uri)
       || (self->priv->proxy_uri && !proxy_uri)
       || (self->priv->proxy_uri && proxy_uri && !soup_uri_equal (self->priv->proxy_uri, proxy_uri)))
     {
-      g_object_set (G_OBJECT (self->priv->soup_session),
-                    SOUP_SESSION_PROXY_URI,
-                    proxy_uri,
-                    NULL);
-
       /* Save internal reference to the proxy */
       if (self->priv->proxy_uri)
         soup_uri_free (self->priv->proxy_uri);
@@ -1320,8 +1375,12 @@ fsp_session_set_http_proxy              (FspSession *self,
       return TRUE;
     }
 
+#ifdef HAVE_LIBSOUP_GNOME
+  return using_gnome_proxy_before != use_gnome_proxy;
+#else
   /* Proxy configuration has not changed */
   return FALSE;
+#endif
 }
 
 const gchar *
@@ -2050,3 +2109,59 @@ fsp_session_get_tags_list_finish        (FspSession    *self,
   return (GSList*) _finish_async_request (G_OBJECT (self), res,
                                           fsp_session_get_tags_list_async, error);
 }
+
+void
+fsp_session_set_license_async           (FspSession          *self,
+                                         const gchar         *photo_id,
+                                         FspLicense          license,
+                                         GCancellable        *cancellable,
+                                         GAsyncReadyCallback  callback,
+                                         gpointer             data)
+{
+  FspSessionPrivate *priv = NULL;
+  SoupSession *soup_session = NULL;
+  gchar *license_str = NULL;
+  gchar *url = NULL;
+  gchar *signed_query = NULL;
+
+  g_return_if_fail (FSP_IS_SESSION (self));
+  g_return_if_fail (photo_id != NULL);
+
+  /* Build the signed url */
+  priv = self->priv;
+  license_str = g_strdup_printf ("%d", license);
+  signed_query = _get_signed_query (priv->secret,
+                                    "method", "flickr.photos.licenses.setLicense",
+                                    "api_key", priv->api_key,
+                                    "auth_token", priv->token,
+                                    "photo_id", photo_id,
+                                    "license_id", license_str,
+                                    NULL);
+  g_free (license_str);
+  url = g_strdup_printf ("%s/?%s", FLICKR_API_BASE_URL, signed_query);
+  g_free (signed_query);
+
+  /* Perform the async request */
+  soup_session = _get_soup_session (self);
+  _perform_async_request (soup_session, url,
+                          _set_license_soup_session_cb, G_OBJECT (self),
+                          cancellable, callback, fsp_session_set_license_async, data);
+
+  g_free (url);
+}
+
+gboolean
+fsp_session_set_license_finish          (FspSession    *self,
+                                         GAsyncResult  *res,
+                                         GError       **error)
+{
+  gpointer result = NULL;
+
+  g_return_val_if_fail (FSP_IS_SESSION (self), FALSE);
+  g_return_val_if_fail (G_IS_ASYNC_RESULT (res), FALSE);
+
+  result = _finish_async_request (G_OBJECT (self), res,
+                                  fsp_session_set_license_async, error);
+
+  return result ? TRUE : FALSE;
+}
