@@ -13,10 +13,8 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>
  *
  */
 
@@ -46,7 +44,9 @@
 
 #define API_KEY "18861766601de84f0921ce6be729f925"
 #define SHARED_SECRET "6233fbefd85f733a"
-#define DEFAULT_TIMEOUT 50
+
+#define DEFAULT_TIMEOUT 100
+#define MAX_AUTH_TIMEOUT 60000
 
 #define FROGR_CONTROLLER_GET_PRIVATE(object)                    \
   (G_TYPE_INSTANCE_GET_PRIVATE ((object),                       \
@@ -67,16 +67,19 @@ struct _FrogrControllerPrivate
   FrogrAccount *account;
 
   FspSession *session;
-  GCancellable *cancellable;
+  GCancellable *last_cancellable;
 
   /* We use this booleans as flags */
   gboolean app_running;
   gboolean uploading_picture;
+  gboolean fetching_auth_url;
+  gboolean fetching_auth_token;
   gboolean fetching_account_info;
   gboolean fetching_account_extra_info;
   gboolean fetching_sets;
   gboolean fetching_groups;
   gboolean fetching_tags;
+  gboolean setting_license;
   gboolean adding_to_set;
   gboolean adding_to_group;
 
@@ -116,6 +119,8 @@ typedef struct {
 
 typedef enum {
   FETCHING_NOTHING,
+  FETCHING_AUTH_URL,
+  FETCHING_AUTH_TOKEN,
   FETCHING_ACCOUNT_INFO,
   FETCHING_ACCOUNT_EXTRA_INFO,
   FETCHING_SETS,
@@ -132,7 +137,9 @@ static void _enable_cancellable (FrogrController *self, gboolean enable);
 static void _notify_error_to_user (FrogrController *self,
                                    GError *error);
 
-static void _show_auth_failed_dialog (GtkWindow *parent, const gchar *message);
+static void _show_auth_failed_dialog (GtkWindow *parent, const gchar *message, gboolean auto_retry);
+
+static void _show_auth_failed_dialog_and_retry (GtkWindow *parent, const gchar *message);
 
 static void _data_fraction_sent_cb (FspSession *session, gdouble fraction, gpointer data);
 
@@ -142,12 +149,18 @@ static void _get_auth_url_cb (GObject *obj, GAsyncResult *res, gpointer data);
 
 static void _complete_auth_cb (GObject *object, GAsyncResult *result, gpointer data);
 
+static gboolean _cancel_authorization_on_timeout (gpointer data);
+
 static void _upload_picture (FrogrController *self, FrogrPicture *picture,
                              FrogrPictureUploadedCallback picture_uploaded_cb,
                              GObject *object);
 
 static void _upload_picture_cb (GObject *object, GAsyncResult *res, gpointer data);
 
+static void _set_license_cb (GObject *object, GAsyncResult *res, gpointer data);
+
+static gboolean _create_set_or_add_picture_on_idle (gpointer data);
+
 static gboolean _create_set_or_add_picture (FrogrController *self,
                                             FrogrPicture *picture,
                                             upload_picture_st *up_st);
@@ -162,6 +175,9 @@ static gboolean _add_picture_to_groups_on_idle (gpointer data);
 
 static gboolean _complete_picture_upload_on_idle (gpointer data);
 
+static void _notify_setting_license (FrogrController *self,
+                                     FrogrPicture *picture);
+
 static void _notify_creating_set (FrogrController *self,
                                   FrogrPicture *picture,
                                   FrogrPhotoSet *set);
@@ -234,7 +250,7 @@ _enable_cancellable (FrogrController *self, gboolean enable)
 {
   FrogrControllerPrivate *priv = FROGR_CONTROLLER_GET_PRIVATE (self);
 
-  priv->cancellable = enable ? g_cancellable_new () : NULL;
+  priv->last_cancellable = enable ? g_cancellable_new () : NULL;
 }
 
 static void
@@ -247,7 +263,7 @@ _notify_error_to_user (FrogrController *self, GError *error)
   switch (error->code)
     {
     case FSP_ERROR_CANCELLED:
-      msg = g_strdup (_("Process cancelled by the user"));
+      msg = g_strdup (_("Process cancelled"));
       error_function = NULL; /* Don't notify the user about this */
       break;
 
@@ -317,8 +333,8 @@ _notify_error_to_user (FrogrController *self, GError *error)
       break;
 
     case FSP_ERROR_AUTHENTICATION_FAILED:
-      msg = g_strdup_printf (_("Authorization failed.\n" "Please try again"));
-      error_function = _show_auth_failed_dialog;
+      msg = g_strdup_printf (_("Authorization failed.\nPlease try again"));
+      error_function = _show_auth_failed_dialog_and_retry;
       break;
 
     case FSP_ERROR_NOT_AUTHENTICATED:
@@ -352,7 +368,7 @@ _notify_error_to_user (FrogrController *self, GError *error)
 }
 
 static void
-_show_auth_failed_dialog (GtkWindow *parent, const gchar *message)
+_show_auth_failed_dialog (GtkWindow *parent, const gchar *message, gboolean auto_retry)
 {
   GtkWidget *dialog = NULL;
 
@@ -365,11 +381,17 @@ _show_auth_failed_dialog (GtkWindow *parent, const gchar *message)
 
   g_signal_connect (G_OBJECT (dialog), "response",
                     G_CALLBACK (_auth_failed_dialog_response_cb),
-                    NULL);
+                    GINT_TO_POINTER ((gint)auto_retry));
 
   gtk_widget_show_all (dialog);
 }
 
+static void
+_show_auth_failed_dialog_and_retry (GtkWindow *parent, const gchar *message)
+{
+  _show_auth_failed_dialog (parent, message, TRUE);
+}
+
 static void
 _data_fraction_sent_cb (FspSession *session, gdouble fraction, gpointer data)
 {
@@ -385,7 +407,8 @@ _data_fraction_sent_cb (FspSession *session, gdouble fraction, gpointer data)
 static void
 _auth_failed_dialog_response_cb (GtkDialog *dialog, gint response, gpointer data)
 {
-  if (response == GTK_RESPONSE_OK)
+  gboolean auto_retry = (gboolean)GPOINTER_TO_INT (data);
+  if (auto_retry && response == GTK_RESPONSE_OK)
     {
       frogr_controller_show_auth_dialog (frogr_controller_get_instance ());
       DEBUG ("%s", "Showing the authorization dialog once again...");
@@ -403,28 +426,30 @@ _get_auth_url_cb (GObject *obj, GAsyncResult *res, gpointer data)
   gchar *auth_url = NULL;
 
   auth_url = fsp_session_get_auth_url_finish (priv->session, res, &error);
+  if (auth_url != NULL && error == NULL)
+    {
+      GtkWindow *window = NULL;
+
+      frogr_util_open_uri (auth_url);
+
+      /* Run the auth confirmation dialog */
+      window = frogr_main_view_get_window (priv->mainview);
+      frogr_auth_dialog_show (window, CONFIRM_AUTHORIZATION);
+
+      DEBUG ("Auth URL: %s", auth_url);
+
+      g_free (auth_url);
+    }
+
   if (error != NULL)
     {
       _notify_error_to_user (self, error);
       DEBUG ("Error getting auth URL: %s", error->message);
       g_error_free (error);
-      return;
     }
 
-  DEBUG ("Auth URL: %s", auth_url ? auth_url : "No URL got");
-
-  /* Open url in the default application */
-  if (auth_url != NULL)
-    {
-      GtkWindow *window = NULL;
-
-      frogr_util_open_url_in_browser (auth_url);
-      g_free (auth_url);
-
-      /* Run the auth confirmation dialog */
-      window = frogr_main_view_get_window (priv->mainview);
-      frogr_auth_dialog_show (window, CONFIRM_AUTHORIZATION);
-    }
+  frogr_main_view_hide_progress (priv->mainview);
+  priv->fetching_auth_url = FALSE;
 }
 
 static void
@@ -432,11 +457,13 @@ _complete_auth_cb (GObject *object, GAsyncResult *result, gpointer data)
 {
   FspSession *session = NULL;
   FrogrController *controller = NULL;
+  FrogrControllerPrivate *priv = NULL;
   FspDataAuthToken *auth_token = NULL;
   GError *error = NULL;
 
   session = FSP_SESSION (object);
   controller = FROGR_CONTROLLER (data);
+  priv = FROGR_CONTROLLER_GET_PRIVATE (controller);
 
   auth_token = fsp_session_complete_auth_finish (session, result, &error);
   if (auth_token)
@@ -466,6 +493,29 @@ _complete_auth_cb (GObject *object, GAsyncResult *result, gpointer data)
       DEBUG ("Authorization failed: %s", error->message);
       g_error_free (error);
     }
+
+  frogr_main_view_hide_progress (priv->mainview);
+  priv->fetching_auth_token = FALSE;
+}
+
+static gboolean
+_cancel_authorization_on_timeout (gpointer data)
+{
+  FrogrController *self = FROGR_CONTROLLER (data);
+  FrogrControllerPrivate *priv = FROGR_CONTROLLER_GET_PRIVATE (self);
+
+  if (priv->fetching_auth_url || priv->fetching_auth_token)
+    {
+      GtkWindow *window = NULL;
+
+      frogr_controller_cancel_ongoing_request (self);
+      frogr_main_view_hide_progress (priv->mainview);
+
+      window = frogr_main_view_get_window (priv->mainview);
+      _show_auth_failed_dialog (window, _("Authorization failed (timed out)\nPlease try again"), FALSE);
+    }
+
+  return FALSE;
 }
 
 static void
@@ -522,7 +572,7 @@ _upload_picture (FrogrController *self, FrogrPicture *picture,
                             safety_level,
                             content_type,
                             search_scope,
-                            priv->cancellable, _upload_picture_cb, up_st);
+                            priv->last_cancellable, _upload_picture_cb, up_st);
 }
 
 static void
@@ -554,23 +604,40 @@ _upload_picture_cb (GObject *object, GAsyncResult *res, gpointer data)
   /* Stop reporting to the user */
   g_signal_handlers_disconnect_by_func (priv->session, _data_fraction_sent_cb, controller);
 
-  /* Check whether it's needed or not to add the picture to the set */
+  /* Check whether it's needed or not to set a specific license for a
+     picture or to add the picture to sets or groups */
   if (!error)
     {
       GSList *sets = NULL;
       GSList *groups = NULL;
+      FspLicense license = FSP_LICENSE_NONE;
 
+      license = frogr_picture_get_license (picture);
       sets = frogr_picture_get_sets (picture);
       groups = frogr_picture_get_groups (picture);
 
+      /* Set license if needed */
+      if (license != FSP_LICENSE_NONE)
+        {
+          priv->setting_license = TRUE;
+
+          _notify_setting_license (controller, picture);
+          fsp_session_set_license_async (session,
+                                         frogr_picture_get_id (picture),
+                                         license,
+                                         priv->last_cancellable,
+                                         _set_license_cb,
+                                         up_st);
+        }
+
+      /* Add picture to set if needed (and maybe create a new one) */
       if (g_slist_length (sets) > 0)
         {
           up_st->sets = sets;
-          _create_set_or_add_picture (controller, picture, up_st);
+          gdk_threads_add_timeout (DEFAULT_TIMEOUT, _create_set_or_add_picture_on_idle, up_st);
         }
 
-      /* Pictures will be added to groups AFTER being added to sets,
-         so that's why we don't start the process now but on idle */
+      /* Add picture to groups if needed */
       if (g_slist_length (groups) > 0)
         {
           up_st->groups = groups;
@@ -583,6 +650,52 @@ _upload_picture_cb (GObject *object, GAsyncResult *res, gpointer data)
   gdk_threads_add_timeout (DEFAULT_TIMEOUT, _complete_picture_upload_on_idle, up_st);
 }
 
+static void
+_set_license_cb (GObject *object, GAsyncResult *res, gpointer data)
+{
+  FspSession *session = NULL;
+  upload_picture_st *up_st = NULL;
+  FrogrController *controller = NULL;
+  FrogrControllerPrivate *priv = NULL;
+  GError *error = NULL;
+
+  session = FSP_SESSION (object);
+  up_st = (upload_picture_st*) data;
+  controller = up_st->controller;
+
+  fsp_session_set_license_finish (session, res, &error);
+  if (error)
+    {
+      /* We do not anything special if something went wrong here */
+      DEBUG ("Error setting license for picture: %s", error->message);
+      g_free (error);
+    }
+
+  priv = FROGR_CONTROLLER_GET_PRIVATE (controller);
+  priv->setting_license = FALSE;
+}
+
+static gboolean
+_create_set_or_add_picture_on_idle (gpointer data)
+{
+  upload_picture_st *up_st = NULL;
+  FrogrController *controller = NULL;
+  FrogrControllerPrivate *priv = NULL;
+  FrogrPicture *picture = NULL;
+
+  up_st = (upload_picture_st*) data;
+  controller = up_st->controller;
+  picture = up_st->picture;
+
+  /* Keep the source while busy */
+  priv = FROGR_CONTROLLER_GET_PRIVATE (controller);
+  if (priv->setting_license)
+    return TRUE;
+
+  _create_set_or_add_picture (controller, picture, up_st);
+  return FALSE;
+}
+
 static gboolean
 _create_set_or_add_picture (FrogrController *self,
                             FrogrPicture *picture,
@@ -612,7 +725,7 @@ _create_set_or_add_picture (FrogrController *self,
           fsp_session_add_to_photoset_async (priv->session,
                                              frogr_picture_get_id (picture),
                                              frogr_photoset_get_id (set),
-                                             priv->cancellable,
+                                             priv->last_cancellable,
                                              _add_to_photoset_cb,
                                              up_st);
         }
@@ -624,7 +737,7 @@ _create_set_or_add_picture (FrogrController *self,
                                              frogr_photoset_get_title (set),
                                              frogr_photoset_get_description (set),
                                              frogr_picture_get_id (picture),
-                                             priv->cancellable,
+                                             priv->last_cancellable,
                                              _create_photoset_cb,
                                              up_st);
         }
@@ -780,7 +893,7 @@ _add_to_group_cb (GObject *object, GAsyncResult *res, gpointer data)
           fsp_session_add_to_group_async (session,
                                           frogr_picture_get_id (picture),
                                           frogr_group_get_id (group),
-                                          priv->cancellable,
+                                          priv->last_cancellable,
                                           _add_to_group_cb,
                                           up_st);
           keep_going = TRUE;
@@ -840,7 +953,7 @@ _add_picture_to_groups_on_idle (gpointer data)
       fsp_session_add_to_group_async (session,
                                       frogr_picture_get_id (picture),
                                       frogr_group_get_id (group),
-                                      priv->cancellable,
+                                      priv->last_cancellable,
                                       _add_to_group_cb,
                                       up_st);
     }
@@ -864,7 +977,7 @@ _complete_picture_upload_on_idle (gpointer data)
   priv = FROGR_CONTROLLER_GET_PRIVATE (controller);
 
   /* Keep the source while busy */
-  if (priv->adding_to_set || priv->adding_to_group)
+  if (priv->setting_license || priv->adding_to_set || priv->adding_to_group)
     {
       frogr_main_view_pulse_progress (priv->mainview);
       return TRUE;
@@ -893,6 +1006,27 @@ _complete_picture_upload_on_idle (gpointer data)
   return FALSE;
 }
 
+static void
+_notify_setting_license (FrogrController *self,
+                         FrogrPicture *picture)
+{
+  FrogrControllerPrivate *priv = NULL;
+  const gchar *picture_title = NULL;
+  FspLicense license = FSP_LICENSE_NONE;
+  gchar *debug_msg = NULL;
+
+  priv = FROGR_CONTROLLER_GET_PRIVATE (self);
+  frogr_main_view_set_progress_description(priv->mainview, _("Setting license for picture…"));
+
+  picture_title = frogr_picture_get_title (picture);
+  license = frogr_picture_get_license (picture);
+  debug_msg = g_strdup_printf ("Setting license %d for picture %s…",
+                               license, picture_title);
+  DEBUG ("%s", debug_msg);
+
+  g_free (debug_msg);
+}
+
 static void
 _notify_creating_set (FrogrController *self,
                       FrogrPicture *picture,
@@ -902,7 +1036,7 @@ _notify_creating_set (FrogrController *self,
   const gchar *picture_title = NULL;
   const gchar *set_title = NULL;
   const gchar *set_desc = NULL;
-  gchar *progress_text = NULL;
+  gchar *debug_msg = NULL;
 
   priv = FROGR_CONTROLLER_GET_PRIVATE (self);
   frogr_main_view_set_progress_description (priv->mainview, _("Creating new photosets…"));
@@ -910,12 +1044,12 @@ _notify_creating_set (FrogrController *self,
   picture_title = frogr_picture_get_title (picture);
   set_title = frogr_photoset_get_title (set);
   set_desc = frogr_photoset_get_description (set);
-  progress_text = g_strdup_printf ("Creating new photoset for picture %s. "
-                                   "Title: %s / Description: %s",
-                                   picture_title, set_title, set_desc);
-  DEBUG ("%s", progress_text);
+  debug_msg = g_strdup_printf ("Creating new photoset for picture %s. "
+                               "Title: %s / Description: %s",
+                               picture_title, set_title, set_desc);
+  DEBUG ("%s", debug_msg);
 
-  g_free (progress_text);
+  g_free (debug_msg);
 }
 
 static void
@@ -926,18 +1060,18 @@ _notify_adding_to_set (FrogrController *self,
   FrogrControllerPrivate *priv = NULL;
   const gchar *picture_title = NULL;
   const gchar *set_title = NULL;
-  gchar *progress_text = NULL;
+  gchar *debug_msg = NULL;
 
   priv = FROGR_CONTROLLER_GET_PRIVATE (self);
   frogr_main_view_set_progress_description(priv->mainview, _("Adding picture to photosets…"));
 
   picture_title = frogr_picture_get_title (picture);
   set_title = frogr_photoset_get_title (set);
-  progress_text = g_strdup_printf ("Adding picture %s to photoset %s…",
-                                   picture_title, set_title);
-  DEBUG ("%s", progress_text);
+  debug_msg = g_strdup_printf ("Adding picture %s to photoset %s…",
+                               picture_title, set_title);
+  DEBUG ("%s", debug_msg);
 
-  g_free (progress_text);
+  g_free (debug_msg);
 }
 
 static void
@@ -948,18 +1082,18 @@ _notify_adding_to_group (FrogrController *self,
   FrogrControllerPrivate *priv = NULL;
   const gchar *picture_title = NULL;
   const gchar *group_name = NULL;
-  gchar *progress_text = NULL;
+  gchar *debug_msg = NULL;
 
   priv = FROGR_CONTROLLER_GET_PRIVATE (self);
   frogr_main_view_set_progress_description(priv->mainview, _("Adding picture to groups…"));
 
   picture_title = frogr_picture_get_title (picture);
   group_name = frogr_group_get_name (group);
-  progress_text = g_strdup_printf ("Adding picture %s to group %s…",
-                                   picture_title, group_name);
-  DEBUG ("%s", progress_text);
+  debug_msg = g_strdup_printf ("Adding picture %s to group %s…",
+                               picture_title, group_name);
+  DEBUG ("%s", debug_msg);
 
-  g_free (progress_text);
+  g_free (debug_msg);
 }
 
 static void
@@ -1038,18 +1172,17 @@ _fetch_everything (FrogrController *self, gboolean force_fetch)
 
   priv = FROGR_CONTROLLER_GET_PRIVATE (self);
 
-  if (force_fetch || !priv->account_info_fetched)
-    _fetch_account_info (self);
-
-  if (force_fetch || !priv->account_extra_info_fetched)
-    _fetch_account_extra_info (self);
+  /* Account information are cheap requests so we always retrieve that
+     info to ensure it's always up to date (regadless 'force_fetch') */
+  _fetch_account_info (self);
+  _fetch_account_extra_info (self);
 
+  /* Sets, groups and tags can take much longer to retrieve, so we
+     only retrieve that if actually needed (or asked to) */
   if (force_fetch || !priv->sets_fetched)
     _fetch_sets (self);
-
   if (force_fetch || !priv->groups_fetched)
     _fetch_groups (self);
-
   if (force_fetch || !priv->tags_fetched)
     _fetch_tags (self);
 }
@@ -1069,7 +1202,7 @@ _fetch_sets (FrogrController *self)
   priv->fetching_sets = TRUE;
 
   _enable_cancellable (self, TRUE);
-  fsp_session_get_photosets_async (priv->session, priv->cancellable,
+  fsp_session_get_photosets_async (priv->session, priv->last_cancellable,
                                    _fetch_sets_cb, self);
 }
 
@@ -1151,7 +1284,7 @@ _fetch_groups (FrogrController *self)
   priv->fetching_groups = TRUE;
 
   _enable_cancellable (self, TRUE);
-  fsp_session_get_groups_async (priv->session, priv->cancellable,
+  fsp_session_get_groups_async (priv->session, priv->last_cancellable,
                                 _fetch_groups_cb, self);
 }
 
@@ -1385,7 +1518,7 @@ _fetch_tags (FrogrController *self)
   priv->fetching_tags = TRUE;
 
   _enable_cancellable (self, TRUE);
-  fsp_session_get_tags_list_async (priv->session, priv->cancellable, _fetch_tags_cb, self);
+  fsp_session_get_tags_list_async (priv->session, priv->last_cancellable, _fetch_tags_cb, self);
 }
 
 static void
@@ -1443,6 +1576,16 @@ _show_progress_on_idle (gpointer data)
   activity = GPOINTER_TO_INT (data);
   switch (activity)
     {
+    case FETCHING_AUTH_URL:
+      text = _("Retrieving data for authorization…");
+      show_dialog = priv->fetching_auth_url;
+      break;
+
+    case FETCHING_AUTH_TOKEN:
+      text = _("Finishing authorization…");
+      show_dialog = priv->fetching_auth_token;
+      break;
+
     case FETCHING_SETS:
       text = _("Retrieving list of sets…");
       show_dialog = priv->fetching_tags;
@@ -1462,9 +1605,14 @@ _show_progress_on_idle (gpointer data)
       text = NULL;
     }
 
-  /* Actually show the dialog, if needed */
+  /* Actually show the dialog and keep the source */
   if (show_dialog)
-    frogr_main_view_show_progress (priv->mainview, text);
+    {
+      frogr_main_view_show_progress (priv->mainview, text);
+      frogr_main_view_pulse_progress (priv->mainview);
+
+      return TRUE;
+    }
 
   return FALSE;
 }
@@ -1485,10 +1633,8 @@ _show_details_dialog_on_idle (GSList *pictures)
 
   /* Keep the source while internally busy */
   if (priv->fetching_tags && frogr_config_get_tags_autocompletion (priv->config))
-    {
-      frogr_main_view_pulse_progress (mainview);
-      return TRUE;
-    }
+    return TRUE;
+
   frogr_main_view_hide_progress (mainview);
 
   mainview_model = frogr_main_view_get_model (priv->mainview);
@@ -1517,10 +1663,8 @@ _show_add_tags_dialog_on_idle (GSList *pictures)
 
   /* Keep the source while internally busy */
   if (priv->fetching_tags && frogr_config_get_tags_autocompletion (priv->config))
-    {
-      frogr_main_view_pulse_progress (mainview);
       return TRUE;
-    }
+
   frogr_main_view_hide_progress (mainview);
 
   mainview_model = frogr_main_view_get_model (priv->mainview);
@@ -1549,10 +1693,8 @@ _show_create_new_set_dialog_on_idle (GSList *pictures)
 
   /* Keep the source while internally busy */
   if (priv->fetching_sets)
-    {
-      frogr_main_view_pulse_progress (mainview);
       return TRUE;
-    }
+
   frogr_main_view_hide_progress (mainview);
 
   mainview_model = frogr_main_view_get_model (priv->mainview);
@@ -1580,10 +1722,8 @@ _show_add_to_set_dialog_on_idle (GSList *pictures)
 
   /* Keep the source while internally busy */
   if (priv->fetching_sets)
-    {
-      frogr_main_view_pulse_progress (mainview);
       return TRUE;
-    }
+
   frogr_main_view_hide_progress (mainview);
 
   mainview_model = frogr_main_view_get_model (priv->mainview);
@@ -1614,10 +1754,8 @@ _show_add_to_group_dialog_on_idle (GSList *pictures)
 
   /* Keep the source while internally busy */
   if (priv->fetching_groups)
-    {
-      frogr_main_view_pulse_progress (mainview);
       return TRUE;
-    }
+
   frogr_main_view_hide_progress (mainview);
 
   mainview_model = frogr_main_view_get_model (priv->mainview);
@@ -1682,10 +1820,10 @@ _frogr_controller_dispose (GObject* object)
       priv->session = NULL;
     }
 
-  if (priv->cancellable)
+  if (priv->last_cancellable)
     {
-      g_object_unref (priv->cancellable);
-      priv->cancellable = NULL;
+      g_object_unref (priv->last_cancellable);
+      priv->last_cancellable = NULL;
     }
 
   G_OBJECT_CLASS (frogr_controller_parent_class)->dispose (object);
@@ -1772,13 +1910,17 @@ frogr_controller_init (FrogrController *self)
   g_object_ref (priv->config);
 
   priv->session = fsp_session_new (API_KEY, SHARED_SECRET, NULL);
-  priv->cancellable = NULL;
+  priv->last_cancellable = NULL;
   priv->app_running = FALSE;
   priv->uploading_picture = FALSE;
+  priv->fetching_auth_url = FALSE;
+  priv->fetching_auth_token = FALSE;
+  priv->fetching_account_info = FALSE;
+  priv->fetching_account_extra_info = FALSE;
   priv->fetching_sets = FALSE;
   priv->fetching_groups = FALSE;
-  priv->fetching_account_extra_info = FALSE;
   priv->fetching_tags = FALSE;
+  priv->setting_license = FALSE;
   priv->adding_to_set = FALSE;
   priv->adding_to_group = FALSE;
   priv->account_info_fetched = FALSE;
@@ -1802,11 +1944,13 @@ frogr_controller_init (FrogrController *self)
   /* Set HTTP proxy if needed */
   if (frogr_config_get_use_proxy (priv->config))
     {
+      const gboolean use_gnome_proxy = frogr_config_get_use_gnome_proxy (priv->config);
       const gchar *host = frogr_config_get_proxy_host (priv->config);
       const gchar *port = frogr_config_get_proxy_port (priv->config);
       const gchar *username = frogr_config_get_proxy_username (priv->config);
       const gchar *password = frogr_config_get_proxy_password (priv->config);
-      frogr_controller_set_proxy (self, host, port, username, password);
+      frogr_controller_set_proxy (self, use_gnome_proxy,
+                                  host, port, username, password);
     }
 }
 
@@ -1992,6 +2136,7 @@ frogr_controller_get_state (FrogrController *self)
 
 void
 frogr_controller_set_proxy (FrogrController *self,
+                            gboolean use_gnome_proxy,
                             const char *host, const char *port,
                             const char *username, const char *password)
 {
@@ -2002,25 +2147,33 @@ frogr_controller_set_proxy (FrogrController *self,
   priv = FROGR_CONTROLLER_GET_PRIVATE (self);
 
   /* The host is mandatory to set up a proxy */
-  if (host == NULL || *host == '\0') {
-    fsp_session_set_http_proxy (priv->session, NULL, NULL, NULL, NULL);
-    DEBUG ("%s", "Not using HTTP proxy");
+  if (!use_gnome_proxy && (host == NULL || *host == '\0')) {
+    fsp_session_set_http_proxy (priv->session, FALSE,
+                                NULL, NULL, NULL, NULL);
+    DEBUG ("%s", "Not enabling the HTTP proxy");
   } else {
-    gboolean has_username = FALSE;
-    gboolean has_password = FALSE;
     gboolean proxy_changed = FALSE;
-    gchar *auth_part = NULL;
 
-    has_username = (username != NULL && *username != '\0');
-    has_password = (password != NULL && *password != '\0');
+    if (!use_gnome_proxy)
+      {
+        gchar *auth_part = NULL;
+        gboolean has_username = FALSE;
+        gboolean has_password = FALSE;
 
-    if (has_username && has_password)
-      auth_part = g_strdup_printf ("%s:%s@", username, password);
+        has_username = (username != NULL && *username != '\0');
+        has_password = (password != NULL && *password != '\0');
 
-    DEBUG ("Using HTTP proxy: %s%s:%s", auth_part ? auth_part : "", host, port);
-    g_free (auth_part);
+        if (has_username && has_password)
+          auth_part = g_strdup_printf ("%s:%s@", username, password);
+
+        DEBUG ("Using HTTP proxy: %s%s:%s", auth_part ? auth_part : "", host, port);
+        g_free (auth_part);
+      }
+    else
+      DEBUG ("Using GNOME general proxy settings");
 
     proxy_changed = fsp_session_set_http_proxy (priv->session,
+                                                use_gnome_proxy,
                                                 host, port,
                                                 username, password);
 
@@ -2204,8 +2357,14 @@ frogr_controller_open_auth_url (FrogrController *self)
 
   g_return_if_fail(FROGR_IS_CONTROLLER (self));
 
-  _enable_cancellable (self, FALSE);
-  fsp_session_get_auth_url_async (priv->session, NULL, _get_auth_url_cb, self);
+  priv->fetching_auth_url = TRUE;
+  _enable_cancellable (self, TRUE);
+
+  fsp_session_get_auth_url_async (priv->session, priv->last_cancellable, _get_auth_url_cb, self);
+  gdk_threads_add_timeout (DEFAULT_TIMEOUT, (GSourceFunc) _show_progress_on_idle, GINT_TO_POINTER (FETCHING_AUTH_URL));
+
+  /* Make sure we show proper feedback if connection is too slow */
+  gdk_threads_add_timeout (MAX_AUTH_TIMEOUT, (GSourceFunc) _cancel_authorization_on_timeout, self);
 }
 
 void
@@ -2215,8 +2374,14 @@ frogr_controller_complete_auth (FrogrController *self)
 
   g_return_if_fail(FROGR_IS_CONTROLLER (self));
 
-  _enable_cancellable (self, FALSE);
-  fsp_session_complete_auth_async (priv->session, NULL, _complete_auth_cb, self);
+  priv->fetching_auth_token = TRUE;
+  _enable_cancellable (self, TRUE);
+
+  fsp_session_complete_auth_async (priv->session, priv->last_cancellable, _complete_auth_cb, self);
+  gdk_threads_add_timeout (DEFAULT_TIMEOUT, (GSourceFunc) _show_progress_on_idle, GINT_TO_POINTER (FETCHING_AUTH_TOKEN));
+
+  /* Make sure we show proper feedback if connection is too slow */
+  gdk_threads_add_timeout (MAX_AUTH_TIMEOUT, (GSourceFunc) _cancel_authorization_on_timeout, self);
 }
 
 gboolean
@@ -2322,9 +2487,10 @@ frogr_controller_cancel_ongoing_request (FrogrController *self)
   g_return_if_fail(FROGR_IS_CONTROLLER (self));
 
   priv = FROGR_CONTROLLER_GET_PRIVATE (self);
-  if (!G_IS_CANCELLABLE (priv->cancellable)
-      || g_cancellable_is_cancelled (priv->cancellable))
+  if (!G_IS_CANCELLABLE (priv->last_cancellable)
+      || g_cancellable_is_cancelled (priv->last_cancellable))
     return;
 
-  g_cancellable_cancel (priv->cancellable);
+  g_cancellable_cancel (priv->last_cancellable);
+  priv->last_cancellable = NULL;
 }
