@@ -13,10 +13,8 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>
  *
  */
 
@@ -47,6 +45,7 @@ typedef struct _FrogrSettingsDialogPrivate {
   GtkWidget *friend_cb;
   GtkWidget *family_cb;
   GtkWidget *show_in_search_cb;
+  GtkWidget *license_cb;
   GtkWidget *photo_content_rb;
   GtkWidget *sshot_content_rb;
   GtkWidget *other_content_rb;
@@ -65,6 +64,7 @@ typedef struct _FrogrSettingsDialogPrivate {
   GtkWidget *proxy_username_entry;
   GtkWidget *proxy_password_label;
   GtkWidget *proxy_password_entry;
+  GtkWidget *use_gnome_proxy_cb;
 
   gboolean public_visibility;
   gboolean family_visibility;
@@ -72,10 +72,13 @@ typedef struct _FrogrSettingsDialogPrivate {
   gboolean show_in_search;
   gboolean disable_tags_autocompletion;
   gboolean keep_file_extensions;
+  FspLicense license;
   FspSafetyLevel safety_level;
   FspContentType content_type;
 
   gboolean use_proxy;
+  gboolean use_gnome_proxy;
+
   gchar *proxy_host;
   gchar *proxy_port;
   gchar *proxy_username;
@@ -85,6 +88,18 @@ typedef struct _FrogrSettingsDialogPrivate {
 
 static FrogrSettingsDialog *_instance = NULL;
 
+static const gchar *license_descriptions[] = {
+  N_("Default (as specified in flickr)"),
+  N_("All rights reserved"),
+  N_("CC Attribution-NonCommercial-ShareAlike"),
+  N_("CC Attribution-NonCommercial"),
+  N_("CC Attribution-NonCommercial-NoDerivs"),
+  N_("CC Attribution"),
+  N_("CC Attribution-ShareAlike"),
+  N_("CC Attribution-NoDerivs"),
+  NULL
+};
+
 /* Prototypes */
 
 static void _add_togleabble_item (FrogrSettingsDialog *self, GtkBox *box,
@@ -95,6 +110,8 @@ static void _add_general_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
 
 static void _add_connection_page (FrogrSettingsDialog *self, GtkNotebook *notebook);
 
+static void _add_misc_page (FrogrSettingsDialog *self, GtkNotebook *notebook);
+
 static void _fill_dialog_with_data (FrogrSettingsDialog *self);
 
 static gboolean _save_data (FrogrSettingsDialog *self);
@@ -103,6 +120,8 @@ static void _update_ui (FrogrSettingsDialog *self);
 
 static void _on_button_toggled (GtkToggleButton *button, gpointer data);
 
+static void _on_combo_changed (GtkComboBox *combo_box, gpointer data);
+
 static void _proxy_port_inserted_cb (GtkEditable *editable, gchar *new_text,
                                      gint new_text_length, gint *position,
                                      gpointer data);
@@ -145,10 +164,17 @@ _add_general_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
   GtkWidget *padding_hbox = NULL;
   GtkWidget *align = NULL;
   GtkWidget *label = NULL;
+  GtkWidget *combo = NULL;
   gchar *markup = NULL;
+  gint i;
 
   priv = FROGR_SETTINGS_DIALOG_GET_PRIVATE (self);
+
+#ifdef GTK_API_VERSION_3
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+#else
   vbox = gtk_vbox_new (FALSE, 6);
+#endif
 
   /* Default Visibility */
 
@@ -163,8 +189,13 @@ _add_general_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
   gtk_container_add (GTK_CONTAINER (align), label);
   gtk_box_pack_start (GTK_BOX (vbox), align, FALSE, FALSE, 6);
 
+#ifdef GTK_API_VERSION_3
+  box1 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+  box2 = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
+#else
   box1 = gtk_vbox_new (FALSE, 6);
   box2 = gtk_hbox_new (FALSE, 12);
+#endif
 
   _add_togleabble_item (self, GTK_BOX (box2), NULL, TRUE, _("_Private"), &priv->private_rb);
   _add_togleabble_item (self, GTK_BOX (box2), GTK_RADIO_BUTTON (priv->private_rb),
@@ -172,14 +203,23 @@ _add_general_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
 
   gtk_box_pack_start (GTK_BOX (box1), box2, FALSE, FALSE, 0);
 
+#ifdef GTK_API_VERSION_3
+  box2 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+#else
   box2 = gtk_vbox_new (FALSE, 6);
+#endif
 
   _add_togleabble_item (self, GTK_BOX (box2), NULL, FALSE,
                         _("Visible to _Family"), &priv->family_cb);
   _add_togleabble_item (self, GTK_BOX (box2), NULL, FALSE,
                         _("Visible to F_riends"), &priv->friend_cb);
 
+#ifdef GTK_API_VERSION_3
+  padding_hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+#else
   padding_hbox = gtk_hbox_new (FALSE, 0);
+#endif
+
   gtk_box_pack_start (GTK_BOX (padding_hbox), box2, FALSE, FALSE, 12);
   gtk_box_pack_start (GTK_BOX (box1), padding_hbox, FALSE, FALSE, 0);
 
@@ -202,7 +242,11 @@ _add_general_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
   gtk_container_add (GTK_CONTAINER (align), label);
   gtk_box_pack_start (GTK_BOX (vbox), align, FALSE, FALSE, 6);
 
+#ifdef GTK_API_VERSION_3
+  box1 = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
+#else
   box1 = gtk_hbox_new (FALSE, 12);
+#endif
 
   _add_togleabble_item (self, GTK_BOX (box1), NULL, TRUE,
                         _("P_hoto"), &priv->photo_content_rb);
@@ -226,7 +270,11 @@ _add_general_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
   gtk_container_add (GTK_CONTAINER (align), label);
   gtk_box_pack_start (GTK_BOX (vbox), align, FALSE, FALSE, 6);
 
+#ifdef GTK_API_VERSION_3
+  box1 = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
+#else
   box1 = gtk_hbox_new (FALSE, 12);
+#endif
 
   _add_togleabble_item (self, GTK_BOX (box1), NULL, TRUE,
                         _("S_afe"), &priv->safe_rb);
@@ -237,12 +285,12 @@ _add_general_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
 
   gtk_box_pack_start (GTK_BOX (vbox), box1, FALSE, FALSE, 0);
 
-  /* Misc */
+  /* License type */
 
   label = gtk_label_new (NULL);
   gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
   markup = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>",
-                                    _("Other options"));
+                                    _("Default License"));
   gtk_label_set_markup (GTK_LABEL (label), markup);
   g_free (markup);
 
@@ -250,16 +298,20 @@ _add_general_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
   gtk_container_add (GTK_CONTAINER (align), label);
   gtk_box_pack_start (GTK_BOX (vbox), align, FALSE, FALSE, 6);
 
-  box1 = gtk_vbox_new (FALSE, 6);
+#if GTK_CHECK_VERSION (2,24,0)
+  combo = gtk_combo_box_text_new ();
+  for (i = 0; license_descriptions[i]; i++)
+    gtk_combo_box_text_insert_text (GTK_COMBO_BOX_TEXT (combo), i, _(license_descriptions[i]));
+#else
+  for (i = 0; license_descriptions[i]; i++)
+    gtk_combo_box_insert_text (GTK_COMBO_BOX (combo), i, _(license_descriptions[i]));
+#endif
+  gtk_box_pack_start (GTK_BOX (vbox), combo, FALSE, FALSE, 0);
+  priv->license_cb = combo;
 
-  _add_togleabble_item (self, GTK_BOX (box1), NULL, FALSE,
-                        _("Disa_ble Tags Auto-Completion"),
-                        &priv->disable_tags_autocompletion_cb);
-  _add_togleabble_item (self, GTK_BOX (box1), NULL, FALSE,
-                        _("_Keep File Extensions in Titles when Loading Pictures"),
-                        &priv->keep_file_extensions_cb);
-
-  gtk_box_pack_start (GTK_BOX (vbox), box1, FALSE, FALSE, 0);
+  g_signal_connect (G_OBJECT (priv->license_cb), "changed",
+                    G_CALLBACK (_on_combo_changed),
+                    self);
 
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
   gtk_notebook_append_page (notebook, vbox, gtk_label_new_with_mnemonic (_("_General")));
@@ -275,13 +327,32 @@ _add_connection_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
   GtkWidget *cbutton = NULL;
   GtkWidget *label = NULL;
   GtkWidget *entry = NULL;
+  gchar *markup = NULL;
 
   priv = FROGR_SETTINGS_DIALOG_GET_PRIVATE (self);
+
+#ifdef GTK_API_VERSION_3
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+#else
   vbox = gtk_vbox_new (FALSE, 6);
+#endif
 
   /* Proxy settings */
 
-  cbutton = gtk_check_button_new_with_mnemonic (_("_Use HTTP Proxy"));
+  label = gtk_label_new (NULL);
+  gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+  markup = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>",
+                                    _("Proxy Settings"));
+  gtk_label_set_markup (GTK_LABEL (label), markup);
+  g_free (markup);
+
+  align = gtk_alignment_new (0, 0, 0, 1);
+  gtk_container_add (GTK_CONTAINER (align), label);
+  gtk_box_pack_start (GTK_BOX (vbox), align, FALSE, FALSE, 6);
+
+  /* Enable proxy */
+
+  cbutton = gtk_check_button_new_with_mnemonic (_("_Enable HTTP Proxy"));
   align = gtk_alignment_new (0, 0, 0, 0);
   gtk_container_add (GTK_CONTAINER (align), cbutton);
   gtk_box_pack_start (GTK_BOX (vbox), align, FALSE, FALSE, 0);
@@ -290,7 +361,6 @@ _add_connection_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
   /* Proxy host */
 
   table = gtk_table_new (2, 4, FALSE);
-  gtk_box_pack_start (GTK_BOX (vbox), table, TRUE, TRUE, 0);
 
   label = gtk_label_new_with_mnemonic (_("_Host:"));
   align = gtk_alignment_new (0, 0, 0, 0);
@@ -363,11 +433,31 @@ _add_connection_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
                     GTK_EXPAND | GTK_FILL, 0, 6, 6);
   priv->proxy_password_entry = entry;
 
+#ifdef HAVE_LIBSOUP_GNOME
+
+  /* Use GNOME General Proxy Settings */
+
+  cbutton = gtk_check_button_new_with_mnemonic (_("_Use GNOME General Proxy Settings"));
+  align = gtk_alignment_new (1, 0, 1, 0);
+  gtk_container_add (GTK_CONTAINER (align), cbutton);
+  gtk_table_attach (GTK_TABLE (table), align, 1, 2, 4, 5,
+                    GTK_EXPAND | GTK_FILL, 0, 6, 6);
+  priv->use_gnome_proxy_cb = cbutton;
+#endif
+
+  gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 0);
+
   /* Connect signals */
   g_signal_connect (G_OBJECT (priv->use_proxy_cb), "toggled",
                     G_CALLBACK (_on_button_toggled),
                     self);
 
+#ifdef HAVE_LIBSOUP_GNOME
+  g_signal_connect (G_OBJECT (priv->use_gnome_proxy_cb), "toggled",
+                    G_CALLBACK (_on_button_toggled),
+                    self);
+#endif
+
   g_signal_connect (G_OBJECT (priv->proxy_port_entry), "insert-text",
                     G_CALLBACK (_proxy_port_inserted_cb),
                     NULL);
@@ -376,6 +466,56 @@ _add_connection_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
   gtk_notebook_append_page (notebook, vbox, gtk_label_new_with_mnemonic (_("Connec_tion")));
 }
 
+static void
+_add_misc_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
+{
+  FrogrSettingsDialogPrivate *priv = NULL;
+  GtkWidget *vbox = NULL;
+  GtkWidget *box = NULL;
+  GtkWidget *align = NULL;
+  GtkWidget *label = NULL;
+  gchar *markup = NULL;
+
+  priv = FROGR_SETTINGS_DIALOG_GET_PRIVATE (self);
+
+#ifdef GTK_API_VERSION_3
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+#else
+  vbox = gtk_vbox_new (FALSE, 6);
+#endif
+
+  /* Misc */
+
+  label = gtk_label_new (NULL);
+  gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+  markup = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>",
+                                    _("Other options"));
+  gtk_label_set_markup (GTK_LABEL (label), markup);
+  g_free (markup);
+
+  align = gtk_alignment_new (0, 0, 0, 1);
+  gtk_container_add (GTK_CONTAINER (align), label);
+  gtk_box_pack_start (GTK_BOX (vbox), align, FALSE, FALSE, 6);
+
+#ifdef GTK_API_VERSION_3
+  box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+#else
+  box = gtk_vbox_new (FALSE, 6);
+#endif
+
+  _add_togleabble_item (self, GTK_BOX (box), NULL, FALSE,
+                        _("Disa_ble Tags Auto-Completion"),
+                        &priv->disable_tags_autocompletion_cb);
+  _add_togleabble_item (self, GTK_BOX (box), NULL, FALSE,
+                        _("_Keep File Extensions in Titles when Loading Pictures"),
+                        &priv->keep_file_extensions_cb);
+
+  gtk_box_pack_start (GTK_BOX (vbox), box, FALSE, FALSE, 0);
+
+  gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
+  gtk_notebook_append_page (notebook, vbox, gtk_label_new_with_mnemonic (_("_Misc")));
+}
+
 static void
 _fill_dialog_with_data (FrogrSettingsDialog *self)
 {
@@ -387,11 +527,15 @@ _fill_dialog_with_data (FrogrSettingsDialog *self)
   priv->family_visibility = frogr_config_get_default_family (priv->config);
   priv->friend_visibility = frogr_config_get_default_friend (priv->config);
   priv->show_in_search = frogr_config_get_default_show_in_search (priv->config);
+  priv->license = frogr_config_get_default_license (priv->config);
   priv->content_type = frogr_config_get_default_content_type (priv->config);
   priv->safety_level = frogr_config_get_default_safety_level (priv->config);
   priv->disable_tags_autocompletion = !frogr_config_get_tags_autocompletion (priv->config);
   priv->keep_file_extensions = frogr_config_get_keep_file_extensions (priv->config);
   priv->use_proxy = frogr_config_get_use_proxy (priv->config);
+#ifdef HAVE_LIBSOUP_GNOME
+  priv->use_gnome_proxy = frogr_config_get_use_gnome_proxy (priv->config);
+#endif
 
   g_free (priv->proxy_host);
   priv->proxy_host = g_strdup (frogr_config_get_proxy_host (priv->config));
@@ -426,6 +570,11 @@ _fill_dialog_with_data (FrogrSettingsDialog *self)
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->show_in_search_cb),
                                 priv->show_in_search);
 
+  if (priv->license >= FSP_LICENSE_NONE && priv->license < FSP_LICENSE_LAST)
+    gtk_combo_box_set_active (GTK_COMBO_BOX (priv->license_cb), priv->license + 1);
+  else
+    gtk_combo_box_set_active (GTK_COMBO_BOX (priv->license_cb), FSP_LICENSE_NONE + 1);
+
   if (priv->content_type == FSP_CONTENT_TYPE_SCREENSHOT)
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->sshot_content_rb), TRUE);
   else if (priv->content_type == FSP_CONTENT_TYPE_OTHER)
@@ -448,6 +597,11 @@ _fill_dialog_with_data (FrogrSettingsDialog *self)
 
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->use_proxy_cb),
                                 priv->use_proxy);
+#ifdef HAVE_LIBSOUP_GNOME
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->use_gnome_proxy_cb),
+                                priv->use_gnome_proxy);
+#endif
+
   if (priv->proxy_host)
     gtk_entry_set_text (GTK_ENTRY (priv->proxy_host_entry), priv->proxy_host);
 
@@ -476,6 +630,7 @@ _save_data (FrogrSettingsDialog *self)
   frogr_config_set_default_friend (priv->config, priv->friend_visibility);
   frogr_config_set_default_show_in_search (priv->config, priv->show_in_search);
 
+  frogr_config_set_default_license (priv->config, priv->license);
   frogr_config_set_default_content_type (priv->config, priv->content_type);
   frogr_config_set_default_safety_level (priv->config, priv->safety_level);
 
@@ -483,6 +638,9 @@ _save_data (FrogrSettingsDialog *self)
   frogr_config_set_keep_file_extensions (priv->config, priv->keep_file_extensions);
 
   frogr_config_set_use_proxy (priv->config, priv->use_proxy);
+#ifdef HAVE_LIBSOUP_GNOME
+  frogr_config_set_use_gnome_proxy (priv->config, priv->use_gnome_proxy);
+#endif
 
   g_free (priv->proxy_host);
   priv->proxy_host = g_strdup (gtk_entry_get_text (GTK_ENTRY (priv->proxy_host_entry)));
@@ -519,20 +677,30 @@ static void
 _update_ui (FrogrSettingsDialog *self)
 {
   FrogrSettingsDialogPrivate *priv = NULL;
+  gboolean using_manual_proxy = FALSE;
 
   priv = FROGR_SETTINGS_DIALOG_GET_PRIVATE (self);
 
-  /* Sensitiveness */
+  /* Sensititveness of default visibility related widgets */
+
   gtk_widget_set_sensitive (priv->friend_cb, !priv->public_visibility);
   gtk_widget_set_sensitive (priv->family_cb, !priv->public_visibility);
-  gtk_widget_set_sensitive (priv->proxy_host_label, priv->use_proxy);
-  gtk_widget_set_sensitive (priv->proxy_host_entry, priv->use_proxy);
-  gtk_widget_set_sensitive (priv->proxy_port_label, priv->use_proxy);
-  gtk_widget_set_sensitive (priv->proxy_port_entry, priv->use_proxy);
-  gtk_widget_set_sensitive (priv->proxy_username_label, priv->use_proxy);
-  gtk_widget_set_sensitive (priv->proxy_username_entry, priv->use_proxy);
-  gtk_widget_set_sensitive (priv->proxy_password_label, priv->use_proxy);
-  gtk_widget_set_sensitive (priv->proxy_password_entry, priv->use_proxy);
+
+  /* Sensititveness of proxy settings related widgets */
+
+#ifdef HAVE_LIBSOUP_GNOME
+  gtk_widget_set_sensitive (priv->use_gnome_proxy_cb, priv->use_proxy);
+#endif
+
+  using_manual_proxy = priv->use_proxy && !priv->use_gnome_proxy;
+  gtk_widget_set_sensitive (priv->proxy_host_label, using_manual_proxy);
+  gtk_widget_set_sensitive (priv->proxy_host_entry, using_manual_proxy);
+  gtk_widget_set_sensitive (priv->proxy_port_label, using_manual_proxy);
+  gtk_widget_set_sensitive (priv->proxy_port_entry, using_manual_proxy);
+  gtk_widget_set_sensitive (priv->proxy_username_label, using_manual_proxy);
+  gtk_widget_set_sensitive (priv->proxy_username_entry, using_manual_proxy);
+  gtk_widget_set_sensitive (priv->proxy_password_label, using_manual_proxy);
+  gtk_widget_set_sensitive (priv->proxy_password_entry, using_manual_proxy);
 }
 
 static void
@@ -621,12 +789,34 @@ _on_button_toggled (GtkToggleButton *button, gpointer data)
   if (GTK_WIDGET (button) == priv->use_proxy_cb)
     {
       priv->use_proxy = active;
-      DEBUG ("Use HTTP proxy: %s", active ? "YES" : "NO");
+      DEBUG ("Enable HTTP Proxy: %s", active ? "YES" : "NO");
     }
 
+#ifdef HAVE_LIBSOUP_GNOME
+  if (GTK_WIDGET (button) == priv->use_gnome_proxy_cb)
+    {
+      priv->use_gnome_proxy = active;
+      DEBUG ("Use GNOME General Proxy Settings: %s", active ? "YES" : "NO");
+    }
+#endif
+
   _update_ui (self);
 }
 
+static void
+_on_combo_changed (GtkComboBox *combo_box, gpointer data)
+{
+  FrogrSettingsDialog *self = NULL;
+  FrogrSettingsDialogPrivate *priv = NULL;
+  gint active_id = 0;
+
+  self = FROGR_SETTINGS_DIALOG (data);
+  priv = FROGR_SETTINGS_DIALOG_GET_PRIVATE (self);
+
+  active_id = gtk_combo_box_get_active (GTK_COMBO_BOX (priv->license_cb));
+  priv->license = (FspLicense) active_id - 1;
+}
+
 static void
 _proxy_port_inserted_cb (GtkEditable *editable, gchar *new_text,
                          gint new_text_length, gint *position,
@@ -671,10 +861,11 @@ static void _dialog_response_cb (GtkDialog *dialog, gint response, gpointer data
   /* Update proxy status */
   if (priv->use_proxy)
     frogr_controller_set_proxy (priv->controller,
+                                priv->use_gnome_proxy,
                                 priv->proxy_host, priv->proxy_port,
                                 priv->proxy_username, priv->proxy_password);
   else
-    frogr_controller_set_proxy (priv->controller, NULL, NULL, NULL, NULL);
+    frogr_controller_set_proxy (priv->controller, FALSE, NULL, NULL, NULL, NULL);
 
   gtk_widget_hide (GTK_WIDGET (self));
 }
@@ -738,6 +929,7 @@ frogr_settings_dialog_init (FrogrSettingsDialog *self)
   priv->friend_cb = NULL;
   priv->family_cb = NULL;
   priv->show_in_search_cb = NULL;
+  priv->license_cb = NULL;
   priv->photo_content_rb = NULL;
   priv->sshot_content_rb = NULL;
   priv->other_content_rb = NULL;
@@ -747,6 +939,7 @@ frogr_settings_dialog_init (FrogrSettingsDialog *self)
   priv->disable_tags_autocompletion_cb = NULL;
   priv->keep_file_extensions_cb = NULL;
   priv->use_proxy_cb = NULL;
+  priv->use_gnome_proxy_cb = NULL;
   priv->proxy_host_label = NULL;
   priv->proxy_host_entry = NULL;
   priv->proxy_port_label = NULL;
@@ -759,11 +952,13 @@ frogr_settings_dialog_init (FrogrSettingsDialog *self)
   priv->family_visibility = FALSE;
   priv->friend_visibility = FALSE;
   priv->show_in_search = FALSE;
+  priv->license = FSP_LICENSE_NONE;
   priv->safety_level = FSP_SAFETY_LEVEL_NONE;
   priv->content_type = FSP_CONTENT_TYPE_NONE;
   priv->disable_tags_autocompletion = FALSE;
   priv->keep_file_extensions = FALSE;
   priv->use_proxy = FALSE;
+  priv->use_gnome_proxy = FALSE;
   priv->proxy_host = NULL;
   priv->proxy_port = NULL;
   priv->proxy_username = NULL;
@@ -785,6 +980,7 @@ frogr_settings_dialog_init (FrogrSettingsDialog *self)
 
   _add_general_page (self, notebook);
   _add_connection_page (self, notebook);
+  _add_misc_page (self, notebook);
 
   /* Connect signals */
   g_signal_connect (G_OBJECT (self), "response",
