@@ -13,10 +13,8 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>
  *
  */
 
@@ -31,7 +29,9 @@ const gchar *frogr_util_get_icons_dir (void);
 
 const gchar *frogr_util_get_locale_dir (void);
 
-void frogr_util_open_url_in_browser (const gchar *url);
+void frogr_util_open_uri (const gchar *uri);
+
+void frogr_util_open_images_in_viewer (GList *uris_list);
 
 void frogr_util_show_info_dialog (GtkWindow *parent, const gchar *message);
 
@@ -39,6 +39,8 @@ void frogr_util_show_warning_dialog (GtkWindow *parent, const gchar *message);
 
 void frogr_util_show_error_dialog (GtkWindow *parent, const gchar *message);
 
+GdkPixbuf *frogr_util_get_corrected_pixbuf (GdkPixbuf *pixbuf, gint max_width, gint max_height);
+
 G_END_DECLS
 
 #endif /* FROGR_UTIL_H */
